@@ -48,15 +48,21 @@ class ObjPrint:
         elif isinstance(obj, FunctionType):
             return f"<function {obj.__name__}>"
 
+        if (not isinstance(exclude, list)) and (not isinstance(exclude, tuple)):
+            raise TypeError("exclude has to be list or tuple")
+
+        if (not isinstance(include, list)) and (not isinstance(include, tuple)):
+            raise TypeError("include has to be list or tuple")
+
         # Otherwise we may need to unpack it. Figure out if we should do that first
         if indent_level >= self.depth:
             return self._get_ellipsis(obj)
 
         if isinstance(obj, list) or isinstance(obj, tuple) or isinstance(obj, set):
-            elems = (f"{self.objstr(val, indent_level + 1)}" for val in obj)
+            elems = (f"{self.objstr(val, indent_level + 1, include=include, exclude=exclude)}" for val in obj)
         elif isinstance(obj, dict):
             elems = (
-                f"{self.objstr(key, indent_level + 1)}: {self.objstr(val, indent_level + 1)}"
+                f"{self.objstr(key, indent_level + 1)}: {self.objstr(val, indent_level + 1, include=include, exclude=exclude)}"
                 for key, val in sorted(obj.items())
             )
         else:
@@ -64,7 +70,11 @@ class ObjPrint:
 
             # If it has __str__ or __repr__ overloaded, honor that
             if obj.__class__.__str__ is not object.__str__ or obj.__class__.__repr__ is not object.__repr__:
-                return str(obj)
+                # Make sure we indent properly
+                s = str(obj)
+                lines = s.split("\n")
+                lines[1:] = [self.add_indent(line, indent_level) for line in lines[1:]]
+                return "\n".join(lines)
             return self._get_custom_object_str(obj, indent_level, include, exclude)
 
         return self._get_pack_str(elems, type(obj), indent_level)
@@ -148,7 +158,12 @@ class ObjPrint:
         if self.elements is None:
             elems = list(elems)
         else:
-            first_elems = [next(elems) for _ in range(self.elements)]
+            first_elems = []
+            try:
+                for _ in range(self.elements):
+                    first_elems.append(next(elems))
+            except StopIteration:
+                pass
             if next(elems, None) is not None:
                 first_elems.append("...")
             elems = first_elems
